#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#define PERIOD 20
int NR_FEEDS = 3;
int SPEED = 62500;
volatile int32_t counter = PERIOD;


void set_stopwatch() {
	//CTC TOP=OCR1A PS=256 OCR1A = 31250/62500
	TCNT0=0x00;
	TCCR1A = 0;
	TCCR1B = 1<< WGM12 | 1 << CS12;
	TIMSK1 = 1 << OCIE1A;
	OCR1A = SPEED;
	ICR1=0;
}

void open() {
	//FPWM TOP=ICR1 PS=256 ICR1 = 1250
	TCNT0=0x00;
	DDRD |= (1 << PD5);
	TCCR1A = 1<<WGM11 | 1<<COM1A1 | 1<<COM1A0;
	TCCR1B = 1<<WGM13 | 1<<WGM12 | 1<< CS12;
	ICR1 = 1250;
	PORTD |= (1 << PD7);
	OCR1A = ICR1 - 52;
	_delay_ms(1000);
	OCR1A = ICR1 - 140;
	_delay_ms(1000);
	DDRD &= ~(1 << PD5);
}
void stop() {
	TIMSK1 = 0;
	PORTD &= ~(1 << PD7);
	TCCR1A = 0;
	TCCR1B = 0;
}
ISR(PCINT3_vect) {
	PORTD &= ~(1 << PD7);
	if((PIND & (1 << PD6)) == 0) {
		int nr_beeps = counter / (PERIOD / 6);
		while(nr_beeps) {
			PORTD |= (1 << PD7);
			_delay_ms(200);
			PORTD &= ~(1 << PD7);
			_delay_ms(100);
			nr_beeps--;
		}
	}
}

ISR(PCINT1_vect) {
	PORTD &= ~(1 << PD7);
	if((PINB & (1 << PB2)) == 0) {
		switch(OCR1A) {
			case 62500:
				OCR1A = 31250;
				break;
			case 31250:
				OCR1A = 62500;
				break;
			default:
				break;
		}
		SPEED = OCR1A;
	}
}

ISR (TIMER1_COMPA_vect) {
	PORTD ^= (1 << PD7);
	counter--;
	if(counter == 0 && NR_FEEDS) {
		open();
		set_stopwatch();
		counter = PERIOD;
		NR_FEEDS--;
	}
}

void IO_init() {
	/* Set pin 7 of port D as output */
	DDRD |= (1 << PD7);

	/* Set in 6 of port D as input(button) */
	DDRD &= ~(1 << PD6);
	PORTD |= (1 << PD6); /* turn on pull-up resistor */

	/* Set in 2 of port B as input(button) */
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);

	/* enable interrupts for PORTD, pin 6(PCINT30) */
	PCICR |= (1 << PCIE3) | (1 << PCIE1);
	PCMSK3 |= (1 << PCINT30);
	PCMSK1 |= (1 << PCINT10);
}

int main() {
	IO_init();
	set_stopwatch();
	sei();
	while (NR_FEEDS){
	}
	stop();
	return 0;
}
